@@ -1,27 +1,29 @@
+from __future__ import annotations
+
 import os
 import os.path
 from inspect import currentframe
 
+_IS_WINDOWS = os.name == 'nt'
+
 
 class T:
-    import typing as _t
-    
-    List = _t.List
+    import typing as t
     
     Path = DirPath = FilePath = str
     Name = DirName = FileName = str
-    Paths = DirPaths = FilePaths = List[Path]
-    Names = DirNames = FileNames = List[Name]
+    Paths = DirPaths = FilePaths = list[Path]
+    Names = DirNames = FileNames = list[Name]
+    Ext = str
     
-    PathFormat = _t.Literal[
+    PathFormat = t.Literal[
         'filepath', 'dirpath', 'path', 'filename', 'dirname', 'name', 'zip',
         'dict', 'list', 'dlist'
     ]
     
-    FinderReturn = _t.Iterator[_t.Tuple[FilePath, FileName]]
+    FinderReturn = t.Iterator[tuple[FilePath, FileName]]
     
-    Suffix = _t.Union[None, str, _t.Tuple[str, ...]]
-    Prefix = Suffix
+    Prefix = Suffix = t.Union[None, str, tuple[str, ...]]
 
 
 def normpath(path: T.Path, force_abspath=False) -> T.Path:
@@ -35,13 +37,23 @@ def normpath(path: T.Path, force_abspath=False) -> T.Path:
         ./a/b/c/../     a/b
     """
     if force_abspath:
-        return os.path.abspath(path).replace('\\', '/')
+        out = os.path.abspath(path)
     else:
-        return os.path.normpath(path).replace('\\', '/')
+        out = os.path.normpath(path)
+    if _IS_WINDOWS:
+        out = out.replace('\\', '/')
+    return out
 
 
 # ------------------------------------------------------------------------------
 
+def get_dirpath(path: T.Path) -> T.DirPath:
+    if os.path.isdir(path):
+        return normpath(path)
+    else:
+        return normpath(os.path.dirname(path))
+
+
 def get_dirname(path: T.Path) -> str:
     """ Return the directory name of path.
     
@@ -76,6 +88,23 @@ def get_filename(path: T.Path, suffix=True, strict=False) -> T.Path:
         return os.path.splitext(name)[0]
 
 
+def split(path: T.Path, separate_ext=False) -> tuple[str, ...]:
+    """
+    return:
+        `tuple[str dirpath, str filename]` or `tuple[str dirpath, str
+            filename_stem, str ext]`.
+        the dirpath is asserted not empty. others may be empty.
+    """
+    path = normpath(path, force_abspath=True)
+    head, tail = os.path.split(path)
+    head = head.rstrip('/')
+    if separate_ext:
+        tail, ext = os.path.splitext(tail)
+        return head, tail, ext
+    else:
+        return head, tail
+
+
 def __get_launch_path() -> T.Path:
     """ Get launcher's filepath.
     
@@ -353,7 +382,7 @@ def currdir() -> T.Path:
     return _get_dir_info_from_caller(caller_frame)
 
 
-def relpath(path: T.Path, ret_abspath=True) -> T.Path:
+def relpath(path: T.Path, force_abspath=True) -> T.Path:
     """ Consider relative path always based on caller's.
     
     References: https://blog.csdn.net/Likianta/article/details/89299937
@@ -366,18 +395,21 @@ def relpath(path: T.Path, ret_abspath=True) -> T.Path:
     else:
         out = os.path.abspath(os.path.join(caller_dir, path))
     
-    if ret_abspath:
+    if force_abspath:
         return normpath(out)
     else:
         return normpath(os.path.relpath(out, os.getcwd()))
 
 
-def _get_dir_info_from_caller(frame) -> T.Path:
+def _get_dir_info_from_caller(frame, ignore_error=False) -> T.Path:
     file = frame.f_globals.get('__file__') \
            or frame.f_code.co_filename
     if file.startswith('<') and file.endswith('>'):
-        print(':v4', 'Unable to get dir info from caller frame! Fallback to '
-                     'use current working directory.')
-        return normpath(os.getcwd())
+        if ignore_error:
+            print(':v4p2', 'Unable to locate directory from caller frame! '
+                           'Fallback using current working directory instead.')
+            return normpath(os.getcwd())
+        else:
+            raise OSError('Unable to locate directory from caller frame!')
     else:
         return normpath(os.path.dirname(file))
